// WinClose.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include "windows.h"
#include "psapi.h"

BOOL g_match = FALSE;

BOOL CALLBACK EnumFunc(HWND hWnd, LPARAM lParam)
{
	DWORD dwProcessId = (DWORD) lParam;

	DWORD dwWndProcessId = 0;
	DWORD dwThreadId = GetWindowThreadProcessId(hWnd, &dwWndProcessId);
	if (dwProcessId == dwWndProcessId) {
		PostMessage(hWnd, WM_CLOSE, 0, 0);
		// There may be several windows associated with a process so keep iterating
		return TRUE;
	}

	return TRUE;
}

int main(int argc, char* argv[])
{
	// Require one argument as input which is the executable filename
	if (2 != argc) {
		printf("%s <exe filename>\n", argv[0]);
		return -1;
	}
	char * exefilename = argv[1];

	// We will need to handle two special cases: Idle process (0) and
	// System process which depends on Windows OS version

	// Get the system process id
	OSVERSIONINFO osvi;
	memset(&osvi, 0, sizeof(OSVERSIONINFO));
	osvi.dwOSVersionInfoSize = sizeof(osvi);
	if (FALSE == GetVersionEx(&osvi)) {
		DWORD dw = GetLastError();
		printf("GetVersionEx returns error: %i\n", dw);
		return -1;
	}
	DWORD dwSystemId = 8;
	if (osvi.dwMajorVersion >= 5)
		dwSystemId = (osvi.dwMinorVersion == 0) ? 2 : 4;

	// Enumerate all the process ids
	DWORD proc_ids[1000]; // Let's hope 1000 process ids is good enough!
	memset(proc_ids, 0, sizeof(proc_ids));
	DWORD cbNeeded = 0;
	if (FALSE == EnumProcesses(proc_ids, sizeof(proc_ids), &cbNeeded)) {
		DWORD dw = GetLastError();
		printf("EnumProcesses returns error: %i\n", dw);
		return -1;
	}

	int count = cbNeeded/sizeof(DWORD);
	for (int i=0; i<count; i++) {
		DWORD dwProcessId = proc_ids[i];

		// Skip the idle process and system process
		if ((0 == dwProcessId) || (dwSystemId == dwProcessId)) {
			continue;
		}

		// Get the process handle from the process id
		HANDLE hProcess = OpenProcess(PROCESS_QUERY_INFORMATION | PROCESS_VM_READ, FALSE, dwProcessId);
		if (NULL != hProcess) {
			// Get the module handle
			HMODULE hModule = 0; // The first module is always the main module
			DWORD cbNeeded = 0;
			if (FALSE == EnumProcessModules(hProcess, &hModule, sizeof(HMODULE), &cbNeeded)) {
				DWORD dw = GetLastError();
				printf("EnumProcessModules returns error: %x\n", dw);
				CloseHandle(hProcess);
				return -1;
			}

			// Get the process filename
			TCHAR szFileName[MAX_PATH+1] = {0};
			if (FALSE == GetModuleBaseName(hProcess, hModule, szFileName, MAX_PATH)) {
				DWORD dw = GetLastError();
				printf("GetModuleFileNameEx returns error: %i\n", dw);
				CloseHandle(hProcess);
				return -1;
			}
			
			// Is the process filename a match?
			if (0 == stricmp(szFileName, exefilename)) {
				printf("Closing window belonging to %s\n", exefilename);
				CloseHandle(hProcess);

				// Enumerate all top level windows
				if (FALSE == EnumWindows(&EnumFunc, (LPARAM) dwProcessId) && !g_match) {
					DWORD dw = GetLastError();
					printf("EnumWindows returns error: %i\n", dw);
					return -1;
				}
				
				return 0;
			}
			CloseHandle(hProcess);
		}
	}

	return 0;
}

